==> t_record_lib-0.1.2.tar.gz: t_record_lib-0.1.2/t_record/utils/utils.py <==
"""Utility functions for t_record."""
import json
import os
import time
import traceback
from pathlib import Path
from typing import Any


def __format_value(val: Any) -> str:
    if val is True:
        return "true"
    elif val is False:
        return "false"
    elif val is None:
        return "null"
    else:
        return json.dumps(val)


def compare_dicts(d1: dict, d2: dict, path: str = "") -> list:
    """Compare two dictionaries and return a list of differences."""
    updates = []

    for key in d1.keys() | d2.keys():
        new_path = f"{path}.{key}" if path else key
        has_old = key in d1
        has_new = key in d2
        v1 = d1.get(key)
        v2 = d2.get(key)

        if isinstance(v1, dict) and isinstance(v2, dict):
            updates.extend(compare_dicts(v1, v2, new_path))
        elif has_old and not has_new:
            updates.append(f"{new_path}: {__format_value(v1)} -> null")
        elif not has_old and has_new:
            updates.append(f"{new_path}: {__format_value(v2)}")
        elif v1 != v2:
            updates.append(f"{new_path}: {__format_value(v1)} -> {__format_value(v2)}")

    return updates


def format_traceback_short(stack: list[traceback.FrameSummary] = None, work_dir: str = None) -> str:
    """Format the traceback for better readability."""
    if stack is None:
        stack = traceback.extract_stack()[:-2]

    work_dir = work_dir or os.getcwd()
    formatted = []

    for frame in stack:
        if "site-packages" in frame.filename or "<frozen" in frame.filename:
            continue

        try:
            rel_path = os.path.relpath(frame.filename, work_dir)
        except ValueError:
            rel_path = frame.filename

        if frame.name != "<module>":
            formatted.append(f"{rel_path}:{frame.lineno} ({frame.name})")
        else:
            formatted.append(f"{rel_path}:{frame.lineno}")

    return "\n".join(formatted)


class SimpleFileLock:
    """A very simple cross-platform file lock."""

    def __init__(self, lock_file_path: Path, retry_delay: float = 0.1, max_retries: int = 300):
        """Initialize the SimpleFileLock."""
        self.lock_file_path = lock_file_path
        self.retry_delay = retry_delay
        self.max_retries = max_retries

    def acquire(self) -> None:
        """Acquire the file lock, retrying if necessary."""
        retries = 0
        while retries < self.max_retries:
            try:
                fd = os.open(str(self.lock_file_path), os.O_CREAT | os.O_EXCL | os.O_RDWR)
                os.close(fd)
                return
            except FileExistsError:
                time.sleep(self.retry_delay)
                retries += 1
        raise TimeoutError(f"Could not acquire lock after {self.max_retries} retries: {self.lock_file_path}")

    def release(self) -> None:
        """Release the file lock by deleting the lock file."""
        try:
            os.unlink(self.lock_file_path)
        except FileNotFoundError:
            pass

    def __enter__(self) -> None:
        """Context manager entry method to acquire the lock."""
        self.acquire()

    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
        """Context manager exit method to release the lock."""
        self.release()

==> t_record_lib-0.2.0.tar.gz: t_record_lib-0.2.0/t_record/utils/utils.py <==
"""Utility functions for t_record."""
import json
import os
import time
import traceback
from pathlib import Path
from typing import Any


def __format_value(val: Any) -> str:
    if val is True:
        return "true"
    elif val is False:
        return "false"
    elif val is None:
        return "null"
    else:
        return json.dumps(val)


def compare_dicts(d1: dict, d2: dict, path: str = "") -> dict:
    """Compare two dictionaries and return a dict of differences with 'from' and 'to' values."""
    updates = {}

    for key in d1.keys() | d2.keys():
        new_path = f"{path}.{key}" if path else key
        has_old = key in d1
        has_new = key in d2
        v1 = d1.get(key)
        v2 = d2.get(key)

        if isinstance(v1, dict) and isinstance(v2, dict):
            nested_updates = compare_dicts(v1, v2, new_path)
            updates.update(nested_updates)
        elif has_old and not has_new:
            updates[new_path] = {"from": v1, "to": None}
        elif not has_old and has_new:
            updates[new_path] = {"from": None, "to": v2}
        elif v1 != v2:
            updates[new_path] = {"from": v1, "to": v2}

    return updates


def format_traceback_short(stack: list[traceback.FrameSummary] = None, work_dir: str = None) -> str:
    """Format the traceback for better readability."""
    if stack is None:
        stack = traceback.extract_stack()[:-2]

    work_dir = work_dir or os.getcwd()
    formatted = []

    for frame in stack:
        if "site-packages" in frame.filename or "<frozen" in frame.filename:
            continue

        try:
            rel_path = os.path.relpath(frame.filename, work_dir)
        except ValueError:
            rel_path = frame.filename

        if frame.name != "<module>":
            formatted.append(f"{rel_path}:{frame.lineno} ({frame.name})")
        else:
            formatted.append(f"{rel_path}:{frame.lineno}")

    return "\n".join(formatted)


class SimpleFileLock:
    """A very simple cross-platform file lock."""

    def __init__(self, lock_file_path: Path, retry_delay: float = 0.1, max_retries: int = 300):
        """Initialize the SimpleFileLock."""
        self.lock_file_path = lock_file_path
        self.retry_delay = retry_delay
        self.max_retries = max_retries

    def acquire(self) -> None:
        """Acquire the file lock, retrying if necessary."""
        retries = 0
        while retries < self.max_retries:
            try:
                fd = os.open(str(self.lock_file_path), os.O_CREAT | os.O_EXCL | os.O_RDWR)
                os.close(fd)
                return
            except FileExistsError:
                time.sleep(self.retry_delay)
                retries += 1
        raise TimeoutError(f"Could not acquire lock after {self.max_retries} retries: {self.lock_file_path}")

    def release(self) -> None:
        """Release the file lock by deleting the lock file."""
        try:
            os.unlink(self.lock_file_path)
        except FileNotFoundError:
            pass

    def __enter__(self) -> None:
        """Context manager entry method to acquire the lock."""
        self.acquire()

    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
        """Context manager exit method to release the lock."""
        self.release()
